/*
  BeNoPPP
  Main file
  (c) 2003 - Sylvain Tertois
*/

#include "BeNoPPPApp.h"
#include <String.h>
#include <stdio.h>
#include <Alert.h>
#include "MainWindow.h"
#include "Preferences.h"
#include <Path.h>
#include <FindDirectory.h>
#include <Message.h>
#include <File.h>

// application MIME type
const char * const c_AppMimeType="application/x-vnd.stertois-BeNoPPP";

// settings file name
static const char * const c_SettingsFile="BeNoPPP";

// constructor
BeNoPPPApp::BeNoPPPApp()
	: BApplication(c_AppMimeType)
{
}

// destructor
BeNoPPPApp::~BeNoPPPApp()
{
}

// called when the application processed all launch messages
void BeNoPPPApp::ReadyToRun()
{
	// create the main window
	new MainWindow;
}

// main application function
int main()
{
	// load the settings
	/// first find the settings folder
	BPath l_PrefsFilePath;
	if (find_directory(B_USER_SETTINGS_DIRECTORY,&l_PrefsFilePath) == B_OK)
	{
		/// add the file name
		l_PrefsFilePath.Append(c_SettingsFile);
		
		/// open the preferences file
		BFile l_PrefsFile(l_PrefsFilePath.Path(),B_READ_ONLY);
		if (l_PrefsFile.InitCheck()==B_OK)
		{
			/// read the contents of the file inside an archive BMessage
			BMessage l_Archive;
			if (l_Archive.Unflatten(&l_PrefsFile) == B_OK)
				/// create a Preferences object from the archive
				g_Preferences = dynamic_cast<Preferences*>(Preferences::Instantiate(&l_Archive));
		}
	}
	if (g_Preferences==NULL)
		/// the preferences couldn't be loaded, use default values
		g_Preferences=new Preferences;
		
	// run the application
	BeNoPPPApp myApp;
	myApp.Run();
	
	// the application quits: save the settings
	/// first find the settings folder
	if (find_directory(B_USER_SETTINGS_DIRECTORY,&l_PrefsFilePath) == B_OK)
	{
		/// add the file name
		l_PrefsFilePath.Append(c_SettingsFile);

		/// open the preferences file
		BFile l_PrefsFile(l_PrefsFilePath.Path(),B_CREATE_FILE|B_WRITE_ONLY);
		if (l_PrefsFile.InitCheck()==B_OK)
		{
			/// archive the preferences in a BMessage
			BMessage l_Archive;
			if (g_Preferences->Archive(&l_Archive) == B_OK)
				/// and save the archive
				l_Archive.Flatten(&l_PrefsFile);
		}
	}
	
	return 0;
}

// displays an error alert when a driver error occurs (called by the *Thread objects)
void ErrorAlert(status_t error)
{
	// construct the error message
	BString l_Message="Error while communicating with the driver: \"";
	l_Message += strerror(error);
	l_Message += "\"";
	
	// and display the alert box
	(new BAlert("Driver Error",l_Message.String(),"Cancel",NULL,NULL,B_WIDTH_AS_USUAL,B_STOP_ALERT))->Go();
}