/*
  BeNoPPP
  Erase Thread
  (c) 2003 - Sylvain Tertois
*/

#include "EraseThread.h"
#include "EraseView.h"
#include "PicList.h"
#include "NoPPPDriver.h"
#include "BeNoPPPApp.h"
#include "MemoryBlock.h"

#include <Messenger.h>
#include <Alert.h>
#include <stdio.h>

// constructor
EraseThread::EraseThread(BMessenger *eraseview, unsigned int iobase, UsePin *pins, bool detect)
	: BLooper("Erase Thread")
{
	m_EraseView=eraseview;
	m_IOBase=iobase;
	m_Pins=pins;
	m_DetectHardware=detect;
	
	Run();
	PostMessage(ET_GO);
}

// destructor
EraseThread::~EraseThread()
{
	delete m_EraseView;
}

// hook function called when a message arrives
void EraseThread::MessageReceived( BMessage *message)
{
	switch (message->what)
	{
		case ET_GO:
			// start the work
			Go();
			break;
		
		default:
			BLooper::MessageReceived(message);
	}
}
	
// one programing cycle (one word)	
void EraseThread::ProgCycle( NoPPPDriver *driver, PicCommand command, unsigned int data)
{
	driver->SendCommand(command);					// send a write command
	driver->SendData(data);									// send the data
	driver->SendCommand(BEGINERASEPROGRAM);	// send an 'Erase and Program' command
	snooze(20000);											// wait 2ms
}

// actual work
void EraseThread::Go()
{
	try		// an exception will be raised if an arror occurs
	{
		// open the driver
		NoPPPDriver l_Driver(m_IOBase);
		l_Driver.SetPins(m_Pins);
		
		// initialisation
		l_Driver.AllPinsLow();
		m_EraseView->SendMessage(EraseView::EV_ERASESTARTED);
		
		// is the hardware present?
		if (m_DetectHardware && !l_Driver.DetectHardware())
		{
			// no
			unsigned int button = (new BAlert("No hardware","The programer hasn't been detected",
				"Proceed Anyway","Cancel",NULL,B_WIDTH_AS_USUAL,
				B_WARNING_ALERT))->Go();
			if (button==1)
			{
				// cancel
				l_Driver.AllPinsLow();
				throw int(-1);
			}
		}
		l_Driver.AllPinsLow();
			
		// wait that the user inserts the PIC
		if ((new BAlert("Insert PIC","Insert the PIC in the programer","OK","Cancel",NULL))->Go() == 1)
			throw int(-1);
		
		// prepare before turning on VPP (25ms wait before and after, to have clean signals while the PIC enters programing mode)
		l_Driver.VddOn();
		l_Driver.DataWritable();
		l_Driver.DataDown();
		l_Driver.ClockDown();
		snooze(25000);
		m_EraseView->SendMessage(EraseView::EV_VPPON);
		l_Driver.VppOn();
		snooze(25000);
		
		// erase PIC
		l_Driver.SendCommand(LOADCONFIG);						// switch to config mode
		l_Driver.SendData(0x3FFF);										// send a dummy config
		for (unsigned int i_Word=0; i_Word<7; i_Word++)
			l_Driver.SendCommand(INCREMENTADD);				// jump to location 2007 (config word)
		l_Driver.SendCommand(BULKERASE1);						// erase procedure
		l_Driver.SendCommand(BULKERASE2);						//  (from Microchip datasheet)
		l_Driver.SendCommand(BEGINERASEPROGRAM);			//
		snooze(20000);													//
		l_Driver.SendCommand(BULKERASE1);						//
		l_Driver.SendCommand(BULKERASE2);						//
		ProgCycle(&l_Driver,ERASEPROGRAM,0x3FFF);				//
		ProgCycle(&l_Driver,ERASEDATA,0x3FFF);					//
		snooze(20000);													// end of procedure
		l_Driver.VppOff();													// turn vpp off
		l_Driver.DataWritable();											// all
		l_Driver.DataDown();												//  pins
		l_Driver.ClockDown();											//  down
		l_Driver.VddOff();

		// finished

		// send a message to the view saying that it is finished
		m_EraseView->SendMessage(EraseView::EV_ERASESTOPPED);

		// alert the user
		(new BAlert("Erase finished","Erase finished","OK"))->Go();
	
		// this is the end of this thread
		PostMessage(B_QUIT_REQUESTED);
	}
	catch (status_t error)
	{
		// driver error
		ErrorAlert(error);													// display an error message
		m_EraseView->SendMessage(EraseView::EV_ERASESTOPPED);
				// say to the view that the work is finished
		PostMessage(B_QUIT_REQUESTED);							// stop the thread
	}
	catch (int i)
	{
		// the erasing has been cancelled by the user
		m_EraseView->SendMessage(EraseView::EV_ERASESTOPPED);
				// say to the view that the work is finished
		PostMessage(B_QUIT_REQUESTED);							// stop the thread
	}	
}
