/*
  BeNoPPP
  Main Window
  (c) 2003 - Sylvain Tertois
*/

#include "MainWindow.h"
#include "AboutView.h"
#include "TestView.h"
#include "ReadView.h"
#include "WriteView.h"
#include "VerifyView.h"
#include "EraseView.h"
#include "PrefsView.h"
#include "PinsView.h"
#include "Preferences.h"
#include <Application.h>
#include <View.h>
#include <MenuField.h>
#include <Menu.h>
#include <MenuItem.h>
#include <StringView.h>
#include <TabView.h>


/* io base addresses */
static const unsigned int c_PPNumberOfAddresses = 4;
static const unsigned int c_PPAddresses[]={0x378,0x278,0x288,0x388};
static const char * const c_PPNames[]={"378 (LPT1)","278 (LPT2)","288","388"};
static const unsigned int c_SPNumberOfAddresses = 4;
static const unsigned int c_SPAddresses[]={0x3F8,0x2F8,0x3E8,0x2E8};
static const char * const c_SPNames[]={"3F8 (COM1)","2F8 (COM2)","3E8 (COM3)","2E8 (COM4)"};

static const float c_XMargin=5;
static const float c_YMargin=5;

// constructor
MainWindow::MainWindow()
	: BWindow(g_Preferences->m_MainWindowRect,"BeNoPPP",B_TITLED_WINDOW,
		B_NOT_ZOOMABLE|B_NOT_RESIZABLE|B_ASYNCHRONOUS_CONTROLS)
{
	// background view
	BView *l_BackView = new BView(Bounds(),NULL,
		B_FOLLOW_ALL_SIDES,B_WILL_DRAW);
	l_BackView->SetViewColor(ui_color(B_MENU_BACKGROUND_COLOR));
	AddChild(l_BackView);
	
	// hardware select
	BMenu *l_HardMenu = new BMenu("name");
	l_HardMenu->SetLabelFromMarked(true);
	m_HardwareField = new BMenuField(
		BRect(c_XMargin,c_YMargin,c_XMargin+130,c_YMargin+10),NULL,"Hardware:",
		l_HardMenu);
	l_BackView->AddChild(m_HardwareField);
	m_HardwareField->ResizeToPreferred();
	m_HardwareField->SetDivider(55);
	/// add the system configurations
	unsigned int l_NumConfs=g_Preferences->m_SystemPinConfigs.CountItems();
	for (unsigned int i_Conf=0; i_Conf<l_NumConfs; i_Conf++)
		AddHardwareConfig((PinsConfig*)g_Preferences->m_SystemPinConfigs.ItemAt(i_Conf));
	l_HardMenu->AddSeparatorItem();
	/// add the user configurations
	l_NumConfs=g_Preferences->m_UserPinConfigs.CountItems();
	for (unsigned int i_Conf=0; i_Conf<l_NumConfs; i_Conf++)
		AddHardwareConfig((PinsConfig*)g_Preferences->m_UserPinConfigs.ItemAt(i_Conf));
	/// select the correct configuration
	if (g_Preferences->m_PinsConfig == NULL)
		SelectConfig(int(0));
	else
		SelectConfig(g_Preferences->m_PinsConfig->m_Name.String());
	
	// base IO address control
	float l_X1=m_HardwareField->Frame().right+c_XMargin;
	BMenu *l_PortMenu = new BMenu("port");
	l_PortMenu->SetLabelFromMarked(true);
	m_PortField = new BMenuField(
		BRect(l_X1,c_YMargin,l_X1+130,c_YMargin+10),NULL,"port:",l_PortMenu);
	l_BackView->AddChild(m_PortField);
	m_PortField->ResizeToPreferred();
	m_PortField->SetDivider(25);

	// vpp status line
	l_X1=m_PortField->Frame().right+c_XMargin;
	float l_X2=Bounds().right-c_XMargin;
	float l_Y1=m_HardwareField->Frame().bottom;
	m_VppStatus = new BStringView(BRect(l_X1,c_YMargin,l_X2,l_Y1-4),
		NULL,"Programer disabled");
	l_BackView->AddChild(m_VppStatus);
	m_VppStatus->SetAlignment(B_ALIGN_CENTER);
	
	// tab view
	BRect l_TabRect=Bounds();
	l_TabRect.top=l_Y1+c_YMargin;
	BTabView *l_TabView = new BTabView(l_TabRect,NULL);
	
	l_TabRect = l_TabView->Bounds();
	l_TabRect.InsetBy(5,5);
	l_TabRect.bottom -= l_TabView->TabHeight();
	
	/// about tab
	BTab *l_Tab = new BTab();
	l_TabView->AddTab(new AboutView(l_TabRect),l_Tab);
	l_Tab->SetLabel("about");

	/// test tab
	l_Tab = new BTab();
	l_TabView->AddTab(new TestView(l_TabRect,this),l_Tab);
	l_Tab->SetLabel("test");
	
	/// read tab
	l_Tab = new BTab();
	l_TabView->AddTab(new ReadView(l_TabRect,this),l_Tab);
	l_Tab->SetLabel("read");

	/// write tab
	l_Tab = new BTab();
	l_TabView->AddTab(new WriteView(l_TabRect,this),l_Tab);
	l_Tab->SetLabel("write");

	/// verify tab
	l_Tab = new BTab();
	l_TabView->AddTab(new VerifyView(l_TabRect,this),l_Tab);
	l_Tab->SetLabel("verify");
	
	/// erase tab
	l_Tab = new BTab();
	l_TabView->AddTab(new EraseView(l_TabRect,this),l_Tab);
	l_Tab->SetLabel("erase");
	
	/// pins tab
	l_Tab = new BTab();
	m_PinsView=new PinsView(l_TabRect,this);
	l_TabView->AddTab(m_PinsView,l_Tab);
	l_Tab->SetLabel("pins");
	
	/// prefs tab
	l_Tab = new BTab();
	l_TabView->AddTab(new PrefsView(l_TabRect,this),l_Tab);
	l_Tab->SetLabel("prefs");
	
	
	l_BackView->AddChild(l_TabView);
	l_TabView->SetTabWidth(B_WIDTH_FROM_WIDEST);
	
	Show();
}

// destructor
MainWindow::~MainWindow()
{
}

// hook function called when a message arrives
void MainWindow::MessageReceived(BMessage *message)
{
	switch(message->what)
	{
		case BNPPP_CHANGEBASE:
		{
			// the user changed the IO base address
			// save the new address in the preferences
			unsigned int ioBase;
			if (message->FindInt32("address",(int32*)&ioBase) == B_OK)
				g_Preferences->m_IOBase=ioBase;
			break;
		}
		
		case BNPPP_CHANGEHARD:
		{
			// the user changed the hardware
			// get the pointer to the Pinsconfig structure
			PinsConfig *l_Config;
			if (message->FindPointer("pinsconfig",(void**)&l_Config) == B_OK)
			{
				g_Preferences->m_PinsConfig = l_Config;
				m_PinsView->PinsConfigChanged();
				RecreatePortMenu(l_Config->m_PortType);
			}
			break;
		}
			
		default:
			BWindow::MessageReceived(message);
	}
}

// the user closed the window
bool MainWindow::QuitRequested()
{
	// save the window position in the preferences
	g_Preferences->m_MainWindowRect=Frame();
	
	// quit
	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}

// display 'Vpp on' in red
void MainWindow::DisplayVppOn()
{
	m_VppStatus->SetHighColor(192,0,0);
	m_VppStatus->SetText("Vpp is ON");
}

// display 'Vpp off' in green
void MainWindow::DisplayVppOff()
{
	m_VppStatus->SetHighColor(0,128,0);
	m_VppStatus->SetText("Vpp is OFF");
}

void MainWindow::AddHardwareConfig(PinsConfig *config)
{
	// first find if a menu entry with this name already exists
	while(m_HardwareField->Menu()->FindItem(config->m_Name.String()) != NULL)
		// yes... add a ' to the name
		config->m_Name += '\'';
		
	BMessage *l_Message = new BMessage(BNPPP_CHANGEHARD);
	l_Message->AddPointer("pinsconfig",config);
	m_HardwareField->Menu()->AddItem(new BMenuItem(config->m_Name.String(),l_Message));
}

void MainWindow::RemoveHardwareConfig(const char *name)
{
	BMenuItem *l_Item = m_HardwareField->Menu()->FindItem(name);
	if ( (l_Item!=NULL) && (m_HardwareField->Menu()->RemoveItem(l_Item)))
		delete l_Item;
}

void MainWindow::RecreatePortMenu(PortType type)
{
	// empty the menu
	BMenuItem *l_Item;
	while ((l_Item=m_PortField->Menu()->RemoveItem(int32(0))) != NULL)
		delete l_Item;
	
	// select the names and IO addresses
	unsigned int l_NumberOfAddresses = 4;
	const unsigned int *l_Addresses;
	const char * const *l_Names;
	switch(type)
	{
		case PARALLEL:
			l_NumberOfAddresses = c_PPNumberOfAddresses;
			l_Addresses=c_PPAddresses;
			l_Names=c_PPNames;
			break;
			
		case SERIAL:
		default:
			l_NumberOfAddresses=c_SPNumberOfAddresses;
			l_Addresses=c_SPAddresses;
			l_Names=c_SPNames;
			break;
	}
	
	// add the items
	unsigned int l_CurrentIOBaseNum=0;
	for (unsigned int i_BaseNum=0; i_BaseNum<l_NumberOfAddresses; i_BaseNum++)
	{
		BMessage *l_Message = new BMessage(BNPPP_CHANGEBASE);
		l_Message->AddInt32("address",l_Addresses[i_BaseNum]);
		m_PortField->Menu()->AddItem(
				new BMenuItem(l_Names[i_BaseNum],l_Message));
		if (l_Addresses[i_BaseNum]==g_Preferences->m_IOBase)
			l_CurrentIOBaseNum=i_BaseNum;
	}
	m_PortField->Menu()->ItemAt(l_CurrentIOBaseNum)->SetMarked(true);
	g_Preferences->m_IOBase=l_Addresses[l_CurrentIOBaseNum];
}

bool MainWindow::ConfigExists(const char *name)
{
	return (m_HardwareField->Menu()->FindItem(name) != NULL);
}

void MainWindow::SelectConfig(int num)
{
	SelectConfig(m_HardwareField->Menu()->ItemAt(num));
}

void MainWindow::SelectConfig(const char *name)
{
	BMenuItem *l_Item = m_HardwareField->Menu()->FindItem(name);
	if (l_Item==NULL)
		SelectConfig(int(0));
	else
		SelectConfig(l_Item);
}

void MainWindow::SelectConfig(BMenuItem *item)
{
	item->SetMarked(true);
	PostMessage(item->Message());
}
