/*
  BeNoPPP
  "New Pins Configuration" Window
  (c) 2003 - Sylvain Tertois
*/

#include "NewPinConfigWin.h"
#include "Preferences.h"
#include "MainWindow.h"

#include <View.h>
#include <MenuField.h>
#include <Menu.h>
#include <MenuItem.h>
#include <TextControl.h>
#include <Button.h>

static const float c_XMargin=5;
static const float c_YMargin=5;

// port types
const char * const c_PortNames[]={"Parallel","Serial"};

// constructor
NewPinConfigWin::NewPinConfigWin(MainWindow *win)
	: BWindow(BRect(100,100,150,150),"New pins configuration",B_FLOATING_WINDOW_LOOK,
		B_MODAL_APP_WINDOW_FEEL,
		B_NOT_ZOOMABLE|B_NOT_RESIZABLE|B_ASYNCHRONOUS_CONTROLS)
{
	m_Type=PARALLEL;
	m_CopyFrom=NULL;
	m_MainWin=win;
	
	// background view
	BView *l_BackView = new BView(Bounds(),NULL,
		B_FOLLOW_ALL_SIDES,B_WILL_DRAW);
	l_BackView->SetViewColor(ui_color(B_MENU_BACKGROUND_COLOR));
	AddChild(l_BackView);

	// name
	float l_X=c_XMargin;
	float l_Y=c_YMargin;
	m_Name = new 	BTextControl(BRect(l_X,l_Y,l_X+105,l_Y+10),NULL,"Name:",
		"",new BMessage(NPCW_CHANGENAME));
	l_BackView->AddChild(m_Name);
	m_Name->ResizeToPreferred();
	m_Name->SetDivider(30);
	m_Name->SetModificationMessage(new BMessage(NPCW_CHANGENAME));
	l_X=m_Name->Frame().right+c_XMargin;
	
	// port type
	BMenu *l_PortMenu= new BMenu("port");
	for (unsigned int i_PortType=0; i_PortType<NUMTYPES; i_PortType++)
	{
		BMessage *l_Message=new BMessage(NPCW_CHANGEPORTTYPE);
		l_Message->AddInt32("type",i_PortType);
		l_PortMenu->AddItem(new BMenuItem(c_PortNames[i_PortType],l_Message));
	}
	l_PortMenu->SetLabelFromMarked(true);
	l_PortMenu->ItemAt(0)->SetMarked(true);
	BMenuField *l_PortField = new BMenuField(BRect(l_X,l_Y,l_X+100,l_Y+10),
		NULL,"port:",l_PortMenu);
	l_BackView->AddChild(l_PortField);
	l_PortField->ResizeToPreferred();
	l_PortField->SetDivider(30);
	l_X=l_PortField->Frame().right+c_XMargin;
	
	// copy from
	BMenu *l_CopyFromMenu = new BMenu("none");
	l_CopyFromMenu->SetLabelFromMarked(true);
	m_CopyFromField = new BMenuField(BRect(l_X,l_Y,l_X+125,l_Y+10),
		NULL,"based on:",l_CopyFromMenu);
	l_BackView->AddChild(m_CopyFromField);
	m_CopyFromField->ResizeToPreferred();
	m_CopyFromField->SetDivider(50);
	l_X=m_CopyFromField->Frame().right+c_XMargin;
	l_Y=m_CopyFromField->Frame().bottom+c_YMargin;
	ChangeCopyFromField(m_Type);
	
	// OK button
	float l_X2=c_XMargin+3;
	m_OK = new BButton(BRect(l_X2,l_Y,l_X2+10,l_Y+10),NULL,"OK",
		new BMessage(NPCW_OK));
	l_BackView->AddChild(m_OK);
	m_OK->MakeDefault(true);
	m_OK->ResizeToPreferred();
	l_X2=m_OK->Frame().right+c_XMargin;
	m_OK->SetEnabled(false);
	
	// Cancel button
	BButton *l_Cancel = new BButton(BRect(l_X2,l_Y,l_X2+10,l_Y+10),NULL,"Cancel",
		new BMessage(B_QUIT_REQUESTED));
	l_BackView->AddChild(l_Cancel);
	l_Cancel->ResizeToPreferred();
	
	l_Y=m_OK->Frame().bottom+c_YMargin+3;
	
	BWindow::ResizeTo(l_X,l_Y);
	Show();
}

// destructor
NewPinConfigWin::~NewPinConfigWin()
{
}

// hook function called when a message arrives
void NewPinConfigWin::MessageReceived(BMessage *message)
{
	switch(message->what)
	{
		case NPCW_CHANGENAME:
		{
			BString l_NewName(m_Name->Text());
			if ((l_NewName.Length()==0) || m_MainWin->ConfigExists(m_Name->Text()))
				m_OK->SetEnabled(false);
			else
				m_OK->SetEnabled(true);
			break;
		}
		
		case NPCW_CHANGEPORTTYPE:
		{
			int32 l_Param;
			if (message->FindInt32("type",&l_Param) == B_OK)
			{
				m_Type=(PortType)l_Param;
				ChangeCopyFromField(m_Type);
			}
			break;
		}
		
		case NPCW_CHANGECPFROM:
		{
			PinsConfig *l_Config;
			if (message->FindPointer("config",(void**)&l_Config) == B_OK)
			{
				m_CopyFrom=l_Config;
			}
			break;
		}
		
		case NPCW_OK:
		{
			PinsConfig *l_NewConfig;
			if (m_CopyFrom==NULL)
			{
				l_NewConfig = new PinsConfig;
				l_NewConfig->m_PortType=m_Type;
			}	
			else
				l_NewConfig = new PinsConfig(m_CopyFrom);
			l_NewConfig->m_Name=m_Name->Text();
			l_NewConfig->m_IsSystem=false;
			g_Preferences->m_UserPinConfigs.AddItem(l_NewConfig);
			m_MainWin->AddHardwareConfig(l_NewConfig);
			
			m_MainWin->SelectConfig(m_Name->Text());
			
			Quit();
		}
		
		default:
			BWindow::MessageReceived(message);
	}
}

void NewPinConfigWin::ChangeCopyFromField(PortType newType)
{
	// first empty the field
	BMenuItem *l_Item;
	while ((l_Item=m_CopyFromField->Menu()->RemoveItem(int32(0))) != NULL)
		delete l_Item;

	// add the first entry
	BMessage *l_Message=new BMessage(NPCW_CHANGECPFROM);
	l_Message->AddPointer("config",NULL);
	BMenuItem *l_NewItem = new BMenuItem("none",l_Message);
	m_CopyFromField->Menu()->AddItem(l_NewItem);
	l_NewItem->SetMarked(true);
	
	m_CopyFromField->Menu()->AddSeparatorItem();
	
	// add the system configurations
	unsigned int l_NumConfigs=(unsigned int)g_Preferences->m_SystemPinConfigs.CountItems();
	for (unsigned int i_Config=0; i_Config<l_NumConfigs; i_Config++)
	{
		PinsConfig *l_Config=(PinsConfig*)g_Preferences->m_SystemPinConfigs.ItemAt(i_Config);
		// only add the configurations with the correct type
		if (l_Config->m_PortType==newType)
		{
			l_Message = new BMessage(NPCW_CHANGECPFROM);
			l_Message->AddPointer("config",l_Config);
			
			m_CopyFromField->Menu()->AddItem(new BMenuItem(
				l_Config->m_Name.String(),l_Message));
		}
	}
	
	m_CopyFromField->Menu()->AddSeparatorItem();
	
	// add the user configurations
	l_NumConfigs=(unsigned int)g_Preferences->m_UserPinConfigs.CountItems();
	for (unsigned int i_Config=0; i_Config<l_NumConfigs; i_Config++)
	{
		PinsConfig *l_Config=(PinsConfig*)g_Preferences->m_UserPinConfigs.ItemAt(i_Config);
		// only add the configurations with the correct type
		if (l_Config->m_PortType==newType)
		{
			l_Message = new BMessage(NPCW_CHANGECPFROM);
			l_Message->AddPointer("config",l_Config);
			
			m_CopyFromField->Menu()->AddItem(new BMenuItem(
				l_Config->m_Name.String(),l_Message));
		}
	}
	
	// the first entry ('none') is selected
	m_CopyFrom=NULL;
}
