/*
  BeNoPPP
  Driver interface
  (c) 2003 - Sylvain Tertois
  Based on NoPPP for Linux 0.3 by Michael A. Covington, Claus Fuetterer and Geoff McCaughan
*/

#ifndef NOPPPDRIVER_H
#define NOPPPDRIVER_H

#include <SupportDefs.h>

// PIC commands
typedef enum _piccommand
{
	LOADCONFIG=0,
	LOADPROGRAM=2,
	READPROGRAM=4,
	INCREMENTADD=6,
	BEGINERASEPROGRAM=8,
	BEGINPROGRAM=24,
	LOADDATA=3,
	READDATA=5,
	BULKERASE1=1,
	BULKERASE2=7,
	ERASEPROGRAM=9,
	ERASEDATA=11
} PicCommand;

typedef struct _usepin UsePin;
class NoPPPDriver {
public:
	// all these functions (including the constructor) throw a status_t exception if the driver returns an error
	NoPPPDriver(unsigned int ioBase=0x378);
		// ioBase = IO Base address for the parallel port
	~NoPPPDriver();
	
	// low level commands
	void DataWritable();		// call this before sending a data bit to the PIC
	void DataReadable();	// call this before reading a data bit from the PIC
	void DataDown();			// changes the bit to write to 0
	void DataUp();			// changes the bit to write to 1
	void ClockDown();		// changes the clock to 0
	void ClockUp();			// changes the clock to 1
	void VppOn();				// enables vpp (-> pic enters programing mode)
	void VppOff();			// disables vpp (-> pic returns to normal mode)
	void VddOn();				// enables vdd (-> pic power on)
	void VddOff();			// disables vdd (-> pic power off)
	bool ReadData();			// read 1 bit
	void AllPinsLow();		// vpp off, data down, clock down, data readable
	void SetPins(UsePin*);	// set the pins to use
	
	// higher level commands
	bool DetectHardware();	// returns true if the programer is detected
	void SendBit(bool data);	// send one bit to the PIC (handles clock and data, you must call DataWritable() before)
	bool ReceiveBit();		// receive one bit from the PIC (handles clock and data, you must call DataReadable() before)
	
	// even higher level commands
	void SendCommand(PicCommand command);	// send 6 bits, lsb first (you don't need to call DataWritable() before)
	void SendData(unsigned int data);	// send 14 bits, lsb first (you don't need to call DataWritable() before)
	unsigned int ReceiveData();	// send 14 bits, lsb first (you don't need to call DataReadable() before)
	
private:
	int m_FileDescriptor;
		// file descriptor for the driver in /dev
};

#endif //NOPPPDRIVER_H
