/*
  BeNoPPP
  Pins configuration
  (c) 2003 - Sylvain Tertois
*/

#include "PinsConfig.h"
#include "ControlCodes.h"
#include "BeNoPPPApp.h"

#include <memory.h>

// name of BMessage entries
static const char * const c_PMName="Name";
static const char * const c_PMPortType="Port Type";
static const char * const c_PMIsSystem="Is System";
static const char * const c_PMPinNumber="Pin Number";
static const char * const c_PMInverted="Inverted";

// default values
static const char * const c_DVName="";
static const PortType c_DVPortType=PARALLEL;
static const bool c_DVIsSystem=false;
static const Pin c_DVPinNumber=P_NONE;
static const int c_DVInverted=0;

PinsConfig::PinsConfig()
{
	m_Name=c_DVName;
	m_PortType=c_DVPortType;
	m_IsSystem=c_DVIsSystem;
	m_Pins = new UsePin[NUMFUNCTIONS];
	
	for (unsigned int i_Function=0; i_Function<NUMFUNCTIONS; i_Function++)
	{
		m_Pins[i_Function].pin = c_DVPinNumber;
		m_Pins[i_Function].inverted=c_DVInverted;
	}
}

PinsConfig::PinsConfig(const PinsConfig *source)
{
	m_Name=source->m_Name;
	m_PortType=source->m_PortType;
	m_IsSystem=source->m_IsSystem;
	m_Pins = new UsePin[NUMFUNCTIONS];
	memcpy(m_Pins,source->m_Pins,NUMFUNCTIONS*sizeof(UsePin));	
}

PinsConfig::PinsConfig( BMessage *archive)
	: BArchivable(archive)
{
	// retreive all the parameters from the archive.
	// if the message entry isn't found, take default value
	
	// name
	if (archive->FindString(c_PMName,&m_Name) != B_OK)
		m_Name=c_DVName;
		
	// port type
	int32 l_Param;
	if (archive->FindInt32(c_PMPortType,&l_Param) == B_OK)
		m_PortType=(PortType)l_Param;
	else
		m_PortType=c_DVPortType;
	
	// is system
	if (archive->FindBool(c_PMIsSystem,&m_IsSystem) != B_OK)
		m_IsSystem=c_DVIsSystem;
	
	// pins
	m_Pins = new UsePin[NUMFUNCTIONS];
	
	for (unsigned int i_Function=0; i_Function<NUMFUNCTIONS; i_Function++)
	{
		// pin number
		if (archive->FindInt32(c_PMPinNumber,i_Function,&l_Param) == B_OK)
			m_Pins[i_Function].pin = (Pin)l_Param;
		else
			m_Pins[i_Function].pin = c_DVPinNumber;
		
		// and inverted status
		if (archive->FindInt32(c_PMInverted,i_Function,&l_Param) == B_OK)
			m_Pins[i_Function].inverted=(int)l_Param;
		else
			m_Pins[i_Function].inverted=c_DVInverted;
	}	
}

PinsConfig::~PinsConfig()
{
	delete m_Pins;
}

// instantiate a new PinsConfig object from archive
BArchivable *PinsConfig::Instantiate(BMessage *archive)
{
	if ( !validate_instantiation(archive,"PinsConfig"))
		return NULL;
	return new PinsConfig(archive);
}

// archive the PinsConfig object into a BMessage
status_t PinsConfig::Archive(BMessage *archive, bool deep) const
{
	archive->AddString("class","PinsConfig");
	archive->AddString("add_on",c_AppMimeType);

	BArchivable::Archive(archive,deep);
	
	archive->AddString(c_PMName,m_Name);
	archive->AddInt32(c_PMPortType,m_PortType);
	archive->AddBool(c_PMIsSystem,m_IsSystem);
	
	for (unsigned int i_Function=0; i_Function<NUMFUNCTIONS; i_Function++)
	{
		archive->AddInt32(c_PMPinNumber,m_Pins[i_Function].pin);
		archive->AddInt32(c_PMInverted,m_Pins[i_Function].inverted);
	}
	
	return B_OK;
}
