/*
  BeNoPPP
  Pins View
  (c) 2003 - Sylvain Tertois
*/

#include "PinsView.h"
#include "NewPinConfigWin.h"
#include "Preferences.h"
#include "MainWindow.h"
#include <Button.h>
#include <MenuField.h>
#include <MenuItem.h>
#include <CheckBox.h>
#include <File.h>
#include <Resources.h>
#include <Node.h>
#include <NodeInfo.h>

// pins names
static const char * const c_PVFunctions[] = {
	"write","clock","mclr","vpp","read","vdd","in1","in2"
	};
static const char * const c_PVPins[] = {
	"(none)",
	"STROBE","D0","D1","D2","D3","D4","D5","D6","D7","AUTOFD","INIT","SELIN",
	"ACK","BUSY","PERROR","SEL","ERROR",
	"TXD","DTR","RTS",
	"CTS","CD","DSR","RI"
	};

// tells if the different functions are inputs (true) or outputs (false)
static const bool c_PVFuncIn[] = {
	false,false,false,false,true,false,false
	};
	
// indexes used to create the menus (first for the parallel port, then for the serial port)
static const int c_PVStartOutput[] = {
	P_STROBE,P_TXD
	};
static const int c_PVEndOutput[] = {
	P_SELIN,P_RTS
	};
static const int c_PVStartInput[] = {
	P_ACK,P_CTS
	};
static const int c_PVEndInput[] = {
	P_ERROR,P_RI
	};

// constructor
PinsView::PinsView(BRect r,MainWindow *mainwin)
	: BView(r,NULL,B_FOLLOW_ALL_SIDES,B_WILL_DRAW)
{
	m_Created=false;
	m_MainWin=mainwin;
}

// destructor
PinsView::~PinsView()
{
}

static const float c_XMargin=5;
static const float c_YMargin=5;
static const float c_MenuWidth=100;

// called when tab is activated
void PinsView::AttachedToWindow()
{
	// are the child views already created?
	if (!m_Created)
	{
		// no. do it
		
		// change the view backgorund color
		SetViewColor(ui_color(B_MENU_BACKGROUND_COLOR));
		
		float l_X=c_XMargin;
		float l_Y=c_YMargin;
		
		// New button
		BButton *l_Button = new BButton(BRect(l_X,l_Y,l_X+10,l_Y+10),NULL,
			"New config",new BMessage(PV_CREATECONFIG));
		AddChild(l_Button);
		l_Button->ResizeToPreferred();
		l_Button->SetTarget(this);
		l_X=l_Button->Frame().right+c_XMargin;
		
		// Delete button
		m_DeleteButton = new BButton(BRect(l_X,l_Y,l_X+10,l_Y+10),NULL,
			"Delete config",new BMessage(PV_DELETECONFIG));
		AddChild(m_DeleteButton);
		m_DeleteButton->ResizeToPreferred();
		m_DeleteButton->SetTarget(this);
		l_X=m_DeleteButton->Frame().right+c_XMargin;
		
		// Resource button (developper version only)
#ifdef RSRC_BUTTON
		l_Button = new BButton(BRect(l_X,l_Y,l_X+10,l_Y+10),NULL,
			"-> rsrc",new BMessage(PV_SAVERSC));
		AddChild(l_Button);
		l_Button->ResizeToPreferred();
		l_Button->SetTarget(this);
#endif RSRC_BUTTON
	
		// now the pins menus
		l_Y=m_DeleteButton->Frame().bottom+4*c_YMargin;
		l_X=4*c_XMargin;
		float l_X2=(Bounds().right+c_XMargin)/2;
		
		// loop for the menus
		for (unsigned int i_Function=0; i_Function<NUMFUNCTIONS; i_Function++)
		{
			// select the X coordinate
			float l_XField;
			if (i_Function%2==0)
				// left
				l_XField=l_X;
			else
				// right
				l_XField=l_X2;
		
			BMenu *l_PinMenu = new BMenu("pin");
			l_PinMenu->SetLabelFromMarked(true);
			m_PinsFields[i_Function]= new BMenuField(
				BRect(l_XField,l_Y,l_XField+c_MenuWidth,l_Y+10),NULL,
				c_PVFunctions[i_Function],l_PinMenu);
			AddChild(m_PinsFields[i_Function]);
			m_PinsFields[i_Function]->ResizeToPreferred();
			m_PinsFields[i_Function]->SetDivider(30);

			// 'inv' checkbox
			BMessage *l_Message = new BMessage(PV_CHANGEINV);
			l_Message->AddInt32("function",i_Function);
			m_PinsInv[i_Function] = new BCheckBox(
				BRect(l_XField+c_MenuWidth+c_XMargin,l_Y,l_XField+200,l_Y+10),
				NULL,"inv",l_Message);
			AddChild(m_PinsInv[i_Function]);
			m_PinsInv[i_Function]->ResizeToPreferred();
			m_PinsInv[i_Function]->SetTarget(this);
			
			// do we need to update the Y?
			if (i_Function%2==1)
				l_Y=m_PinsFields[i_Function]->Frame().bottom+c_YMargin;
		}
		
		// the view is now constructed
		m_Created=true;
		
		// create the menus
		PinsConfigChanged();
	}
}

// hook function called when a message arrives
void PinsView::MessageReceived(BMessage *message)
{
	switch (message->what)
	{
		case PV_CREATECONFIG:
			new NewPinConfigWin(m_MainWin);
			break;
			
		case PV_DELETECONFIG:
			// remove the configuration from the user list
			if (g_Preferences->m_UserPinConfigs.RemoveItem(g_Preferences->m_PinsConfig))
			{
				m_MainWin->RemoveHardwareConfig(g_Preferences->m_PinsConfig->m_Name.String());
				m_MainWin->SelectConfig(int(0));
			}
			break;
			
		case PV_CHANGEPIN:
		{
			int32 l_Function,l_Pin;
			if ((message->FindInt32("function",&l_Function) == B_OK) &&
					(message->FindInt32("pin",&l_Pin) == B_OK))
				g_Preferences->m_PinsConfig->m_Pins[l_Function].pin = (Pin)l_Pin;
			break;
		}

		case PV_CHANGEINV:
		{
			int32 l_Function;
			if (message->FindInt32("function",&l_Function) == B_OK)
				g_Preferences->m_PinsConfig->m_Pins[l_Function].inverted = 
					m_PinsInv[l_Function]->Value();
			break;
		}
		
		case PV_SAVERSC:
			SaveResource();
			break;
						
		default:
			BView::MessageReceived(message);
	}
}

void PinsView::PinsConfigChanged()
{
	// if the view isn't already created, the view will send itself a message to do this later
	if (!m_Created)
	{
		return;
	}

	// loop on the functions
	for (unsigned int i_Function=0; i_Function<NUMFUNCTIONS; i_Function++)
	{
		// get the menu
		BMenu *l_Menu = m_PinsFields[i_Function]->Menu();
		
		// empty it
		BMenuItem *l_Item;
		while ((l_Item=l_Menu->RemoveItem(int32(0))) != NULL)
			delete l_Item;
			
		// get the pin to select
		Pin l_PinToSelect = g_Preferences->m_PinsConfig->m_Pins[i_Function].pin;
		
		// first add to the menu the 'none' item
		l_Item = AddMenuEntry((Function)i_Function,P_NONE);
		if (l_PinToSelect==P_NONE)
			l_Item->SetMarked(true);
		
		// find which other ones to add
		unsigned int l_StartIndex=0, l_EndIndex=0;
		if (c_PVFuncIn[i_Function])
		{
			// input
			l_StartIndex=c_PVStartInput[g_Preferences->m_PinsConfig->m_PortType];
			l_EndIndex=c_PVEndInput[g_Preferences->m_PinsConfig->m_PortType];
		}
		else
		{
			// input
			l_StartIndex=c_PVStartOutput[g_Preferences->m_PinsConfig->m_PortType];
			l_EndIndex=c_PVEndOutput[g_Preferences->m_PinsConfig->m_PortType];
		}
		
		// and add the items
		for (unsigned int i_Pin=l_StartIndex; i_Pin<=l_EndIndex; i_Pin++)
		{
			l_Item = AddMenuEntry((Function)i_Function, (Pin)i_Pin);
			if ((Pin)i_Pin == l_PinToSelect)
				l_Item->SetMarked(true);
		}
		
		// change the state of the 'inv' checkbox
		m_PinsInv[i_Function]->SetValue(
			g_Preferences->m_PinsConfig->m_Pins[i_Function].inverted);
		
		// enable/disable the controls if it is a system or user config
		m_PinsFields[i_Function]->SetEnabled(
			!g_Preferences->m_PinsConfig->m_IsSystem);
		m_PinsInv[i_Function]->SetEnabled(
			!g_Preferences->m_PinsConfig->m_IsSystem);
		m_DeleteButton->SetEnabled(!g_Preferences->m_PinsConfig->m_IsSystem);
	}
}

BMenuItem *PinsView::AddMenuEntry(Function function, Pin pin)
{
	// create the message
	BMessage *l_Message = new BMessage(PV_CHANGEPIN);
	l_Message->AddInt32("function",function);
	l_Message->AddInt32("pin", pin);
	
	// create the item
	BMenuItem *l_Item = new BMenuItem(c_PVPins[pin],l_Message);
	m_PinsFields[function]->Menu()->AddItem(l_Item);
	l_Item->SetTarget(this);
	
	return l_Item;
}

void PinsView::SaveResource()
{
	// file name
	BString l_FileName(g_Preferences->m_PinsConfig->m_Name.String());
	l_FileName.Append(".rsrc");
	
	// create the resource file
	BFile l_DestFile(l_FileName.String(),
		B_READ_WRITE|B_CREATE_FILE|B_ERASE_FILE);
	BResources l_Resources(&l_DestFile,true);
	
	// create and flatten the archive of the configuration
	BMessage l_Archive('Pins');
	g_Preferences->m_PinsConfig->Archive(&l_Archive,true);
	ssize_t l_FlattenedSize = l_Archive.FlattenedSize();
	char *l_FlattenedArchive = new char[l_FlattenedSize];
	l_Archive.Flatten(l_FlattenedArchive,l_FlattenedSize);
	
	// put the flattened archive in the resource file
	l_Resources.AddResource('Pins',0,l_FlattenedArchive,
		l_FlattenedSize,"pins config");
		
	// free the allocated memory block
	delete[] l_FlattenedArchive;
	
	// set the file type
	BNode l_ResNode(l_FileName.String());
	BNodeInfo l_ResNodeInfo(&l_ResNode);
	l_ResNodeInfo.SetType("application/x-be-resource");
	l_ResNodeInfo.SetPreferredApp("application/x-vnd.Be.QuickRes");
}
