/*
  BeNoPPP
  Preferences class
  (c) 2003 - Sylvain Tertois
*/

#include "Preferences.h"
#include "BeNoPPPApp.h"
#include "PicList.h"
#include "PinsConfig.h"
#include <Application.h>
#include <Resources.h>
#include <Roster.h>

// global Preferences instantiation
Preferences *g_Preferences=NULL;

// name of BMessage entries
static const char * const c_PMMainWinRect="Main Window Rect";
static const char * const c_PMIOBase="IO Base";
static const char * const c_PMDetectHardware="Detect hardware";
static const char * const c_PMHexPath="Hex Path";
static const char * const c_PMHexFormat="Hex format";
static const char * const c_PMVerifyWhenWrite="Verify when write";
static const char * const c_PMPicType="Pic type";
static const char * const c_PMUserPinConfig="User pin configuration";
static const char * const c_PMCurrentConfig="Current pin configuration";

// default values
static const BRect c_DVMainWinRect(150,150,550,450);
static const unsigned int c_DVIOBase=0x378;
static const bool c_DVDetectHardware=true;
static const char *c_DVHexPath=B_EMPTY_STRING;
static const HexFormat c_DVHexFormat=INHX8M;
static const bool c_DVVerifyWhenWrite=true;
static const unsigned int c_DVPicTypeNum=0;
static PinsConfig *c_DVCurrentConfig=NULL;

Preferences::Preferences()
{
	// put some default values
	m_MainWindowRect=c_DVMainWinRect;
	m_IOBase=c_DVIOBase;
	m_DetectHardware=c_DVDetectHardware;
	m_HexPath=c_DVHexPath;
	m_HexFormat=c_DVHexFormat;
	m_VerifyWhenWrite=c_DVVerifyWhenWrite;
	m_PicTypeNum=c_DVPicTypeNum;
	m_PinsConfig=c_DVCurrentConfig;
	
	// load system pin configurations
	LoadSystemConfigs("__none");
}

Preferences::Preferences( BMessage *archive)
	: BArchivable(archive)
{
	// retreive all the parameters from the archive.
	// if the message entry isn't found, take default value
	
	// window size and position
	if (archive->FindRect(c_PMMainWinRect,&m_MainWindowRect) != B_OK)
		m_MainWindowRect=c_DVMainWinRect;
		
	// IO base address
	if (archive->FindInt32(c_PMIOBase,(int32 *)&m_IOBase) != B_OK)
		m_IOBase=c_DVIOBase;
		
	// detect hardware
	if (archive->FindBool(c_PMDetectHardware,&m_DetectHardware) != B_OK)
		m_DetectHardware=c_DVDetectHardware;
		
	// default hex file path
	if (archive->FindString(c_PMHexPath,&m_HexPath) != B_OK)
		m_HexPath=c_DVHexPath;
	
	// default hex file format
	int32 l_Param;
	if ((archive->FindInt32(c_PMHexFormat, &l_Param)==B_OK) &&
			(l_Param>=0) && (l_Param<NUMFORMATS))
		m_HexFormat=(HexFormat)l_Param;
	else
		m_HexFormat=c_DVHexFormat;
	
	// verify when write
	if (archive->FindBool(c_PMVerifyWhenWrite,&m_VerifyWhenWrite) != B_OK)
		m_VerifyWhenWrite=c_DVVerifyWhenWrite;
	
	// default PIC type
	m_PicTypeNum=c_DVPicTypeNum;
	BString l_PicName;
	if (archive->FindString(c_PMPicType,&l_PicName) == B_OK)
	{
		// find the pic type num in our list
		for (unsigned int i_PicNum=0; c_PicList[i_PicNum].name != NULL; i_PicNum++)
			if (l_PicName==c_PicList[i_PicNum].name)
			{
				// found
				m_PicTypeNum=i_PicNum;
				break;
			}
	}
	
	// pin configuration
	m_PinsConfig=NULL;
	const char *l_PinConfigName="__none";
	archive->FindString(c_PMCurrentConfig,&l_PinConfigName);
	
	LoadSystemConfigs(l_PinConfigName);
	BMessage l_ConfArchive;
	unsigned int i_Config=0;
	while(archive->FindMessage(c_PMUserPinConfig,i_Config++,&l_ConfArchive)==B_OK)
	{
		PinsConfig *l_PinsConfig = new PinsConfig(&l_ConfArchive);
		if (l_PinsConfig != NULL)
		{
			m_UserPinConfigs.AddItem(l_PinsConfig);
			if (l_PinsConfig->m_Name==l_PinConfigName)
				m_PinsConfig=l_PinsConfig;
		}
	}
}

Preferences::~Preferences()
{
}

// instantiate a new Preferences object from archive
BArchivable *Preferences::Instantiate(BMessage *archive)
{	
	if ( !validate_instantiation(archive,"Preferences"))
		return NULL;
	return new Preferences(archive);
}

// archive the Preferences object into a BMessage
status_t Preferences::Archive(BMessage *archive, bool deep) const
{
	archive->AddString("class","Preferences");
	archive->AddString("add_on",c_AppMimeType);
	
	BArchivable::Archive(archive,deep);
	
	archive->AddRect(c_PMMainWinRect,m_MainWindowRect);
	archive->AddInt32(c_PMIOBase,m_IOBase);
	archive->AddBool(c_PMDetectHardware,m_DetectHardware);
	archive->AddString(c_PMHexPath,m_HexPath);
	archive->AddInt32(c_PMHexFormat,m_HexFormat);
	archive->AddBool(c_PMVerifyWhenWrite,m_VerifyWhenWrite);
	archive->AddString(c_PMPicType,c_PicList[m_PicTypeNum].name);
	archive->AddString(c_PMCurrentConfig,m_PinsConfig->m_Name);
	
	for (unsigned int i_Config=0; i_Config<((unsigned int)m_UserPinConfigs.CountItems()); i_Config++)
	{
		BMessage l_ConfArchive('UsrC');
		if (((PinsConfig*)(m_UserPinConfigs.ItemAt(i_Config)))->Archive(&l_ConfArchive) == B_OK)
			archive->AddMessage(c_PMUserPinConfig,&l_ConfArchive);
	}
	
	return B_OK;
}

void Preferences::LoadSystemConfigs(const char *nametoselect)
{
	// get the application info to find the resource data
	app_info l_TheAppInfo;
	be_app->GetAppInfo(&l_TheAppInfo);
	BFile l_AppFile(&l_TheAppInfo.ref, B_READ_ONLY);
	BResources l_Resources(&l_AppFile);
	
	// loop to find the resources
	for (unsigned int i_ID=0; l_Resources.HasResource('Pins',i_ID); i_ID++)
	{
		// recreate the archive
		const char *l_Data;
		size_t l_Size;
		l_Data = (const char*)l_Resources.LoadResource('Pins',i_ID,&l_Size);
		if (l_Data != NULL)
		{
			BMessage l_Archive;
			l_Archive.Unflatten(l_Data);
			
			// create the PinsConfig object from the archive
			PinsConfig *l_NewConfig = new PinsConfig(&l_Archive);
			l_NewConfig->m_IsSystem=true;

			// and add it to the menu
			m_SystemPinConfigs.AddItem(l_NewConfig);
			
			// check if this is the default configuration
			if (l_NewConfig->m_Name == nametoselect)
				m_PinsConfig=l_NewConfig;
		}
	}
}
