/*
  BeNoPPP
  Read View
  (c) 2003 - Sylvain Tertois
*/

#include "ReadView.h"
#include "MainWindow.h"
#include "PicList.h"
#include "ReadThread.h"
#include "Preferences.h"
#include <Button.h>	
#include <TextControl.h>
#include <FilePanel.h>
#include <Path.h>
#include <MenuField.h>
#include <Menu.h>
#include <MenuItem.h>
#include <TextView.h>
#include <StatusBar.h>

// constructor
ReadView::ReadView(BRect r, MainWindow *win)
	: BView(r,NULL,B_FOLLOW_ALL_SIDES,B_WILL_DRAW)
{
	m_MainWindow = win;
	m_Created=false;
	m_Panel=NULL;
}

// destructor
ReadView::~ReadView()
{
	delete m_Panel;
}

static const float c_XMargin=5;
static const float c_YMargin=5;

// hook function called when the tab is activated
void ReadView::AttachedToWindow()
{
	if (!m_Created)
	{
		// first call, we must add the child views
		// first the file panel
		m_Panel = new BFilePanel(B_SAVE_PANEL,new BMessenger(this),NULL,
			B_FILE_NODE,false);

		// and then the view
		SetViewColor(ui_color(B_MENU_BACKGROUND_COLOR));
		
		// filename
		float l_X,l_Y;
		float l_Dummy,l_Height;
		/// first the "..." button
		m_FileButton = new BButton(BRect(0,c_YMargin,0,c_YMargin),NULL,"...",
			new BMessage(RV_FILEPANEL));
		AddChild(m_FileButton);
		m_FileButton->SetTarget(this);
		m_FileButton->GetPreferredSize(&l_Dummy,&l_Height);
		m_FileButton->ResizeTo(30,l_Height);
		l_X=Bounds().right-c_XMargin-m_FileButton->Frame().right;
		m_FileButton->MoveTo(l_X,c_YMargin);
		l_X-=c_XMargin;
		l_Y=m_FileButton->Frame().bottom+c_YMargin;
		
		/// and then the text field
		m_FileName = new BTextControl(BRect(c_XMargin,c_YMargin,l_X,c_YMargin),
			NULL,"File name:","",NULL);
		AddChild(m_FileName);
		m_FileName->SetTarget(this);
		m_FileName->SetModificationMessage(new BMessage(RV_FILENAMECHANGED));
		m_FileName->GetPreferredSize(&l_Dummy,&l_Height);
		m_FileName->ResizeTo(l_X-c_XMargin,l_Height);
		m_FileName->SetDivider(50);

		// PIC type menu
		BMenu *l_Menu = new BMenu("pic");
		for (unsigned int i_PicTypeNum=0; c_PicList[i_PicTypeNum].name != NULL; i_PicTypeNum++)
		{
			BMessage *l_Message = new BMessage(RV_PICTYPECHANGED);
			l_Message->AddInt32("pictypenum",i_PicTypeNum);
			l_Menu->AddItem(new BMenuItem(c_PicList[i_PicTypeNum].name,l_Message));
		}
		l_Menu->SetLabelFromMarked(true);
		l_Menu->SetTargetForItems(this);
		m_PicTypeField = new BMenuField(BRect(c_XMargin,l_Y,c_XMargin+150,c_YMargin),
			NULL,"Pic type:",l_Menu);
		AddChild(m_PicTypeField);
		m_PicTypeField->ResizeToPreferred();
		l_X=m_PicTypeField->Frame().right+c_XMargin;
		
		// file format menu
		l_Menu = new BMenu("format");
		for (unsigned int i_FNum=0; i_FNum<NUMFORMATS; i_FNum++)
		{
			BMessage *l_Message = new BMessage(RV_FORMATCHANGED);
			l_Message->AddInt32("format",i_FNum);
			l_Menu->AddItem(new BMenuItem(MemoryBlock::GetFormatName((HexFormat)i_FNum),l_Message));
		}
		l_Menu->SetLabelFromMarked(true);
		l_Menu->SetTargetForItems(this);
		m_FileFormatField = new BMenuField(BRect(l_X,l_Y,l_X+150,c_YMargin),
			NULL,"File format:",l_Menu);
		AddChild(m_FileFormatField);
		m_FileFormatField->ResizeToPreferred();
		l_Y=m_FileFormatField->Frame().bottom+c_YMargin;
		
		// message
		l_X=Bounds().right-c_XMargin;
		BTextView *l_TextView = new BTextView(BRect(c_XMargin,l_Y,l_X,l_Y+40),
			NULL,BRect(0,0,l_X-c_XMargin,40),B_FOLLOW_ALL_SIDES);
		l_TextView->SetText("Warning: do not insert the PIC in the programer before "
			"clicking on the button below!\n"
			"Insert it only when told to do so.");
		AddChild(l_TextView);
		l_TextView->SetViewColor(ui_color(B_MENU_BACKGROUND_COLOR));
		l_TextView->ResizeToPreferred();
		l_TextView->MakeEditable(false);
		l_TextView->MakeSelectable(false);
		l_Y=l_TextView->Frame().bottom+3*c_YMargin;

		// the 'Start' button		
		m_Start = new BButton(BRect(c_XMargin,l_Y,c_XMargin,l_Y),NULL,"Start",
			new BMessage(RV_START));
		AddChild(m_Start);
		m_Start->SetTarget(this);
		m_Start->SetEnabled(false);
		m_Start->ResizeToPreferred();
		l_Y=m_Start->Frame().bottom+c_YMargin;
		
		// the status bar
		m_StatusBar = new BStatusBar(BRect(c_XMargin,l_Y,Bounds().right-c_XMargin,l_Y),
			NULL,"Progress:","Idle");
		AddChild(m_StatusBar);
		m_StatusBar->GetPreferredSize(&l_Dummy,&l_Height);
		m_StatusBar->ResizeTo(Bounds().right-2*c_XMargin,l_Height);
			
		m_Created=true;
	}
	
	// this part of the function is executed each time the tab is activated. We update the controls in case they have been modified elsewhere
	// set the panel directory
	m_Panel->SetPanelDirectory(g_Preferences->m_HexPath.String());

	// set the pic type
	m_PicTypeField->Menu()->ItemAt(g_Preferences->m_PicTypeNum)->SetMarked(true);
	
	// set the file format
	m_FileFormatField->Menu()->ItemAt((int32)g_Preferences->m_HexFormat)->SetMarked(true);
}

// hook function called when a message arrives
void ReadView::MessageReceived(BMessage *message)
{
	switch (message->what)
	{
		case RV_FILEPANEL:
			// open the panel
			m_Panel->Show();
			break;
		
		case B_SAVE_REQUESTED:
		{
			// the user closed the panel and confirmed the file name
			// first get the full file path
			entry_ref l_Dir;
			const char *l_Name;
			if ((message->FindRef("directory",&l_Dir) == B_OK) &&
				(message->FindString("name",&l_Name) == B_OK))
			{
				BPath l_DirPath(&l_Dir);
				BPath l_FilePath(l_DirPath.Path(),l_Name);
				
				// got it. change the text in the control
				m_FileName->SetText(l_FilePath.Path());
				m_MainWindow->PostMessage(RV_FILENAMECHANGED,this);
				
				// save the directory in the preferences
				g_Preferences->m_HexPath=l_DirPath.Path();
			}
			break;
		}
		
		case RV_PICTYPECHANGED:
			// the user changed the pic type
			// save the new type in the preferences
			message->FindInt32("pictypenum",(int32*)&g_Preferences->m_PicTypeNum);
			break;
		
		case RV_FORMATCHANGED:
		{
			// the user changed the file format
			// save the new format in the preferences
			int32 l_Format;
			message->FindInt32("format",(int32*)&l_Format);
			g_Preferences->m_HexFormat=(HexFormat)l_Format;
			break;
		}

		case RV_FILENAMECHANGED:
			// the user changed the file name
			// we can now activate the 'start' button, if it hasn't been already done
			m_Start->SetEnabled(true);
			break;
			
		case RV_START:
			// the user clicked on 'start'
			// start a new thread that will do the work
			new ReadThread(new BMessenger(this),g_Preferences->m_PicTypeNum,
				g_Preferences->m_HexFormat,m_FileName->Text(),
				g_Preferences->m_IOBase,g_Preferences->m_PinsConfig->m_Pins,
				g_Preferences->m_DetectHardware);
			break;
		
		case RV_READSTARTED:
			// the work just started. Vpp isn't on yet
			m_MainWindow->DisplayVppOff();
			break;
		
		case RV_VPPON:
			// vpp is now on
			m_MainWindow->DisplayVppOn();
			break;
		
		case RV_READSTOPPED:
			// the work is finished
			m_MainWindow->DisplayVppOff();
			break;
		
		case RV_STATUSMAX:
		{
			// the thread tells us how many times it will ask to update the status bar
			unsigned int l_MaxValue;
			if (message->FindInt32("max",(int32*)&l_MaxValue)==B_OK)
			{
				// reset the bar and put the "progress:" text
				m_StatusBar->Reset("Progress:");
				
				// set the max value to the one received
				m_StatusBar->SetMaxValue(l_MaxValue);
			}
			break;
		}
		
		case RV_STATUSUPDATE:
		{
			// called to show the progression of the work
			// if a text is inserted in the message, show it as well
			const char *l_UpdateText=NULL;
			message->FindString("text",&l_UpdateText);
			m_StatusBar->Update(1,NULL,l_UpdateText);
			break;
		}
			
		default:
			BView::MessageReceived(message);
	}
}
	
