/*
  BeNoPPP
  Test View
  (c) 2003 - Sylvain Tertois
*/

#include "TestView.h"
#include "MainWindow.h"
#include "BeNoPPPApp.h"
#include "NoPPPDriver.h"
#include "Preferences.h"
#include "ControlCodes.h"
#include <TextView.h>
#include <Button.h>
#include <ScrollView.h>

// constructor
TestView::TestView(BRect r, MainWindow *win)
	: BView(r,NULL,B_FOLLOW_ALL_SIDES,B_WILL_DRAW)
{
	m_MainWindow = win;
	m_Driver=NULL;
	m_Created=false;
}

// destructor
TestView::~TestView()
{
	if (m_Driver != NULL)
	{
		m_Driver->AllPinsLow();
		delete m_Driver;
	}
}

static const float c_XMargin=5;
static const float c_YMargin=5;

// hook function called when the tab is activated
void TestView::AttachedToWindow()
{
	if (!m_Created)
	{
		// first call, we must add the child views
		// first set the view color
		SetViewColor(ui_color(B_MENU_BACKGROUND_COLOR));
		
		// add the 3 buttons
		float l_Y,l_X;
		
		/// start
		m_StartButton = new BButton(BRect(0,0,0,0),NULL,"Start",new BMessage(TV_START));
		AddChild(m_StartButton);
		m_StartButton->ResizeToPreferred();
		//// we move the button so that it fits at the bottom of the window
		l_Y=Bounds().bottom-c_YMargin-m_StartButton->Bounds().bottom;
		m_StartButton->MoveTo(c_XMargin,l_Y);
		l_X=m_StartButton->Frame().right+c_XMargin;
		m_StartButton->SetTarget(this);
		
		/// next
		m_NextButton = new BButton(BRect(l_X,l_Y,l_X,l_Y),NULL,"Next >>",
			new BMessage(TV_NEXT));
		AddChild(m_NextButton);
		m_NextButton->ResizeToPreferred();
		m_NextButton->SetTarget(this);
		m_NextButton->SetEnabled(false);
		l_X=m_NextButton->Frame().right+c_XMargin;
		
		/// stop
		m_StopButton = new BButton(BRect(l_X,l_Y,l_X,l_Y),NULL,"Stop",
			new BMessage(TV_STOP));
		AddChild(m_StopButton);
		m_StopButton->ResizeToPreferred();
		m_StopButton->SetTarget(this);
		m_StopButton->SetEnabled(false);
		
		// text view
		l_Y-= c_YMargin;
		BRect l_ViewRect=Bounds();
		l_ViewRect.InsetBy(5,5);
		l_ViewRect.bottom=l_Y;
		l_ViewRect.right -= B_V_SCROLL_BAR_WIDTH;
		BRect l_TextRect(l_ViewRect);
		l_TextRect.OffsetTo(0,0);
		l_TextRect.InsetBy(2,2);
		
		m_TextView = new BTextView(l_ViewRect,NULL,l_TextRect,
			B_FOLLOW_ALL_SIDES,B_WILL_DRAW);
		AddChild(new BScrollView(NULL,m_TextView,B_FOLLOW_ALL,0,false,true));
		m_TextView->MakeEditable(false);
		m_TextView->SetText("This panel lets you test your programer\n"
			"You'll need a voltmeter. Just clic on 'start' to begin the test, and follow the instructions. "
			"Don't forget to clic on 'stop' before using your programer\n"
			"You must ensure that the programer is powered up, and no PIC is in the socket.\n"
			"You'll need to find the GND, VDD, VPP, B6 and B7 pins. In a 18 pin socket, GND is pin 5, "
			"VDD is pin14, VPP 4, B6 12 and B7 13. In a 28 pin socket, GND is pin 8/19, VDD is pin 20, VPP 1, "
			"B6 27, and B7 28. Finally you'll also need to know how to check if the programmer is in "
			"read or write mode. This depends on the type of programmer. Check the documentation "
			"and schematic of your programmer and/or the BeNoppp documentation");
	
		m_Created=true;
	}
}

// number of tests
static const unsigned int c_NumTests=8;

// hook function called when a message arrives
void TestView::MessageReceived(BMessage *message)
{
	try
	{
		switch (message->what)
		{
			case TV_START:
				// must start the test
				
				// first change the buttons states
				m_StartButton->SetEnabled(false);
				m_NextButton->SetEnabled(true);
				m_StopButton->SetEnabled(true);
				
				// open the driver and proceed to first stage (0)
				m_Driver = new NoPPPDriver(g_Preferences->m_IOBase);
				m_Driver->SetPins(g_Preferences->m_PinsConfig->m_Pins);
				m_StageNumber=0;
				// there isn't a 'break' here, it's not a bug
			
			case TV_NEXT:
				// called to go to a new stage
				if (m_Driver != NULL)
				{
					switch (m_StageNumber)
					{
						case 0:
							// stage 0: hardware detection
							if (m_Driver->DetectHardware())
								m_TextView->SetText("The programer has been detected. Click on 'next' to continue");
							else
								m_TextView->SetText("The programer has not been detected. There could be a problem with it, or it can come from your parallel port. Click on 'next' to continue the test, or 'stop' and 'start' again to do another detection");
							break;
						
						case 1:
							// stage 1: test with all pins low
						{
							BString l_Message("TEST A: all pins low\n"
								"Connect negative voltmeter lead to GND "
								"of PIC socket and check the following voltages:\n");
							// check if programmer is able to change the state of Vdd
							if (g_Preferences->m_PinsConfig->m_Pins[F_VDD].pin==P_NONE)
								l_Message += "  VDD	4.75 to 5.25 V\n";
							else
								l_Message += "  VDD	< 0.8V\n";
								
							l_Message += "  VPP		< 0.8 V\n"
								"  B6		< 0.8 V\n"
								"  B7		< 0.8 V\n"
								"  Programmer should be in write mode";

							m_TextView->SetText(l_Message.String());
							m_Driver->AllPinsLow();
							m_MainWindow->DisplayVppOff();
							break;
						}
													
						case 2:
							// stage 2: Vdd on
							// check if programmer is able to change the state of Vdd
							if (g_Preferences->m_PinsConfig->m_Pins[F_VDD].pin==P_NONE)
								// no... go directly to next stage
								m_StageNumber++;
							else
							{							
								m_TextView->SetText("TEST A2: vdd on\n"
									"With negative voltmeter lead still connected to GND,"
									"check the following voltages:\n"
									"  VDD	4.75 to 5.25 V\n"
									"  VPP		< 0.8 V\n"
									"  B6		< 0.8 V\n"
									"  B7		< 0.8 V\n"
									"  Programmer should be in write mode");
								m_Driver->VddOn();
							break;
							}
						
						case 3:
							// stage 3: Vpp
							m_TextView->SetText("TEST B: vpp on\n"
								"With negative voltmeter lead still connected to GND,"
								"check the following voltages:\n"
								"  VDD	4.75 to 5.25 V\n"
								"  VPP		12 to 14 V\n"
								"  B6		< 0.8 V\n"
								"  B7		< 0.8 V\n"
								"  Programmer should be in write mode");
							
							m_Driver->VppOn();
							m_MainWindow->DisplayVppOn();
							break;
						
						case 4:
							// stage 4: Clock (B6)
							m_TextView->SetText("TEST C: clock\n"
								"With negative voltmeter lead still connected to GND,"
								"check the following voltages:\n"
								"  VDD	4.75 to 5.25 V\n"
								"  VPP		12 to 14 V\n"
								"  B6		> 4 V\n"
								"  B7		< 0.8 V\n"
								"  Programmer should be in write mode");
							
							m_Driver->ClockUp();
							break;

						case 5:
							// stage 5: Data (B7)
							m_TextView->SetText("TEST D: data\n"
								"With negative voltmeter lead still connected to GND,"
								"check the following voltages:\n"
								"  VDD	4.75 to 5.25 V\n"
								"  VPP		12 to 14 V\n"
								"  B6		< 0.8 V\n"
								"  B7		> 4 V\n"
								"  Programmer should be in write mode");
							
							m_Driver->ClockDown();
							m_Driver->DataUp();
							break;
							
						case 6:
							// stage 6: Read mode
							m_TextView->SetText("TEST E: read mode\n"
								"With negative voltmeter lead still connected to GND,"
								"check the following voltages:\n"
								"  VDD	4.75 to 5.25 V\n"
								"  VPP		12 to 14 V\n"
								"  B6		< 0.8 V\n"
								"  B7		? (should be pulled up >4V by a resistor)\n"
								"  Programmer should now be in read mode\n"
								"When you click on 'next', I'll check if I read a 1");
							
							m_Driver->DataReadable();
							break;
							
						case 7:
							// stage 7: read a 1
							m_Driver->DataReadable();
							if (!m_Driver->ReadData())
							{
								m_TextView->SetText("TEST F: read a 1\n"
								"TEST FAILED\n "
								"I should read a 1 and I'm reading a 0...\n"
								"Reading isn't working properly");
								m_StageNumber=c_NumTests;
							}
							else
							{
								m_TextView->SetText("TEST F: read a 1\n"
								"TEST PASSED\n"
								"The first read test passed correctly. Now use a wire or low value"
								" resistor to connect the GND and B7 pins, and press 'Next'");
							}
							break;
							
						case 8:
							// stage 8: read a 0
							m_Driver->DataReadable();
							if (m_Driver->ReadData())
							{
								m_TextView->SetText("TEST G: read a 0\n"
								"TEST FAILED\n "
								"I should read a 0 and I'm reading a 1...\n"
								"Reading isn't working properly");
								m_StageNumber=c_NumTests;
							}
							else
							{
								m_TextView->SetText("TEST G: read a 0\n"
								"TEST PASSED\n"
								"The test is now finished!\n"
								"Click on 'next' to shut the programmer down");
							}
							break;
							
						default:
							// unknown stage number... stop the test
							m_MainWindow->PostMessage(TV_STOP,this);
					}
					m_StageNumber++;
				}
				else
					// there is a problem, the driver should be here
					m_MainWindow->PostMessage(TV_STOP,this);
				break;
				
			case TV_STOP:
				// called when the test is ended or interrupted
				// close the driver
				if (m_Driver != NULL)
				{
					m_Driver->AllPinsLow();
					m_MainWindow->DisplayVppOff();
					delete m_Driver;
					m_Driver=NULL;
				}
				// change the buttons states
				m_StartButton->SetEnabled(true);
				m_NextButton->SetEnabled(false);
				m_StopButton->SetEnabled(false);
				
				// change the text displayed in the text view
				if (m_StageNumber>=c_NumTests)
					m_TextView->SetText("Test finished.");
				else
					m_TextView->SetText("Test interrupted.");
				
				break;
				
			default:
				BView::MessageReceived(message);
		}
	}
	catch (status_t error)
	{
		// called when there is an error while communicating with the driver (see NoPPPDriver.h/cpp)
		ErrorAlert(error);
		delete m_Driver;
		m_Driver = NULL;
		
		m_StartButton->SetEnabled(true);
		m_NextButton->SetEnabled(false);
		m_StopButton->SetEnabled(false);			

		m_TextView->SetText("Test interrupted (driver error).");
	}
}
	
	