/*
  BeNoPPP
  Write Thread
  (c) 2003 - Sylvain Tertois
*/

#include "VerifyThread.h"
#include "VerifyView.h"
#include "PicList.h"
#include "NoPPPDriver.h"
#include "BeNoPPPApp.h"
#include "MemoryBlock.h"

#include <Messenger.h>
#include <Alert.h>
#include <stdio.h>

// constructor
VerifyThread::VerifyThread(BMessenger *verifyview, unsigned int picType,const char *fileName, unsigned int iobase, UsePin *pins, bool detect)
	: BLooper("Verify Thread")
{
	m_VerifyView=verifyview;
	m_PicType=picType;
	m_FileName=fileName;
	m_IOBase=iobase;
	m_Pins=pins;
	m_DetectHardware=detect;
	
	Run();
	PostMessage(VT_GO);
}

// destructor
VerifyThread::~VerifyThread()
{
	delete m_VerifyView;
}

// hook function called when a message arrives
void VerifyThread::MessageReceived( BMessage *message)
{
	switch (message->what)
	{
		case VT_GO:
			Go();
			break;
		
		default:
			BLooper::MessageReceived(message);
	}
}

// called by Go() with the status bar needs an update
void VerifyThread::Update(const char *text)
{
	BMessage l_Message(VerifyView::VV_STATUSUPDATE);
	if (text != NULL)
		l_Message.AddString("text",text);
	
	m_VerifyView->SendMessage(&l_Message);
}
	
// actual work
void VerifyThread::Go()
{
	try		// an exception will be raised if an arror occurs
	{
		// open the driver and allocate a memory block to load the file
		NoPPPDriver l_Driver(m_IOBase);
		l_Driver.SetPins(m_Pins);
		MemoryBlock l_Memory;

		// load the file
		l_Memory.ReadFromFile(m_FileName);
				
		// count how many update messages will be sent
		/// first, get the size of the program and data banks
		l_Memory.SetMemoryBank(INSTRUCTION);
		unsigned int l_ProgramSize=l_Memory.GetBankSize();
		l_Memory.SetMemoryBank(DATA);
		unsigned int l_DataSize=l_Memory.GetBankSize();
		
		/// build the count message and send it to the view
		BMessage l_UpdateSizeMessage(VerifyView::VV_STATUSMAX);
		l_UpdateSizeMessage.AddInt32("max",
			l_ProgramSize+4+1+l_DataSize+4);
		m_VerifyView->SendMessage(&l_UpdateSizeMessage);
		
		// initialisation
		l_Driver.AllPinsLow();
		m_VerifyView->SendMessage(VerifyView::VV_VERIFYSTARTED);
		
		// is the hardware present?
		if (m_DetectHardware && !l_Driver.DetectHardware())
		{
			// no
			unsigned int button = (new BAlert("No hardware","The programer hasn't been detected",
				"Proceed Anyway","Cancel",NULL,B_WIDTH_AS_USUAL,
				B_WARNING_ALERT))->Go();
			if (button==1)
			{
				// cancel
				l_Driver.AllPinsLow();
				throw int(-1);
			}
		}
		l_Driver.AllPinsLow();
			
		// wait that the user inserts the PIC
		if ((new BAlert("Insert PIC","Insert the PIC in the programer","OK","Cancel",NULL))->Go() == 1)
			throw int(-1);
		
		// prepare before turning on VPP (25ms wait before and after, to have clean signals while the PIC enters programing mode)
		l_Driver.VddOn();
		l_Driver.DataWritable();
		l_Driver.DataDown();
		l_Driver.ClockDown();
		snooze(25000);
		m_VerifyView->SendMessage(VerifyView::VV_VPPON);
		l_Driver.VppOn();
		snooze(25000);
			
		// VERIFY			
		// program memory
		Update("verifying program memory");
		l_Memory.SetMemoryBank(INSTRUCTION);			// switch to the correct bank
		for (unsigned int i_Add=0; i_Add<l_ProgramSize; i_Add++)
		{
			unsigned int l_Word = l_Memory.ReadWord() & 0x3FFF;
					// read the word from memory
			l_Driver.SendCommand(READPROGRAM);		// read the word from the PIC
			if (l_Driver.ReceiveData() != l_Word)				// compare the two
			{
				// verify error
				l_Driver.VppOff();									// turn off VPP
				l_Driver.AllPinsLow();								// put all pins low
				throw int(i_Add);									// raise a verify exception
			}
			l_Driver.SendCommand(INCREMENTADD);		// go to next address
			Update();												// update the status bar
		}
		
		// configuration memory (ID)
		Update("verifing id memory");
		l_Memory.SetMemoryBank(CONFIGURATION);		// switch to the correct bank
		l_Driver.SendCommand(LOADCONFIG);				// turn the PIC into config mode
		l_Driver.SendData(c_PicList[m_PicType].defaultConfig);
				// load default config (is this really good?)
		for (unsigned int i_Add=0; i_Add<4; i_Add++)
		{
			unsigned int l_Word = l_Memory.ReadWord() & 0x3FFF;
					// read the word from memory
			l_Driver.SendCommand(READPROGRAM);		// read the word from the PIC
			if (l_Driver.ReceiveData() != l_Word)				// compare the two
			{
				// verify error
				l_Driver.VppOff();									// turn off VPP
				l_Driver.AllPinsLow();								// put all pins low
				throw int(i_Add+0x2000);						// raise a verify exception
			}
			l_Driver.SendCommand(INCREMENTADD);		// go to next address
			Update();												// update the status bar
		}
		
		// goto config word (0x2007)
		// 3 jumps
		for (unsigned int i_Jump=0; i_Jump<3; i_Jump++)
		{
			l_Memory.ReadWord();
			l_Driver.SendCommand(INCREMENTADD);
		}
		
		// verify config word
		Update("verifying configuration word");
		unsigned int l_Word = l_Memory.ReadWord() & 0x3FFF;
					// read the word from memory
		l_Driver.SendCommand(READPROGRAM);			// read the word from the PIC
		if (l_Driver.ReceiveData() != l_Word)					// compare the two
		{
			// verify error
			l_Driver.VppOff();										// turn off VPP
			l_Driver.AllPinsLow();									// put all pins low
			throw int(0x2007);									// raise a verify exception
		}
		l_Driver.SendCommand(INCREMENTADD);			// go to next address
		Update();													// update the status bar

		// reset the PIC to write data memory
		l_Driver.VppOff();											// turn VPP off
		l_Driver.DataWritable();									// put all
		l_Driver.DataDown();										//  pins
		l_Driver.ClockDown();									//  down
		snooze(25000);											// wait 25ms
		l_Driver.VppOn();											// turn VPP on again
		snooze(25000);											// wait 25ms
		
		// verify data memory
		l_Memory.SetMemoryBank(DATA);						// switch to the correct bank
		Update("verifying data memory");
		for (unsigned int i_Add=0; i_Add<l_DataSize; i_Add++)
		{
			unsigned int l_Word = l_Memory.ReadWord() & 0xFF;
					// read the word from memory
			l_Driver.SendCommand(READDATA);				// read the word from the PIC
			if (l_Driver.ReceiveData() != l_Word)				// compare the two
			{
				// verify error
				l_Driver.VppOff();									// turn off VPP
				l_Driver.AllPinsLow();								// put all pins low
				throw int(i_Add+0x2100);						// raise a verify exception
			}
			l_Driver.SendCommand(INCREMENTADD);		// go to next address
			Update();												// update the status bar
		}
				
		// finished
		l_Driver.VppOff();											// turn VPP off
		l_Driver.DataWritable();									// put
		l_Driver.DataDown();										// all pins
		l_Driver.ClockDown();									// down
		l_Driver.VddOff();

		// send a message to the view saying that it is finished
		m_VerifyView->SendMessage(VerifyView::VV_VERIFYSTOPPED);

		// tell the user
		(new BAlert("Verify finished","Verify finished, and ok!","OK"))->Go();
	
		// this is the end of this thread
		PostMessage(B_QUIT_REQUESTED);
	}
	catch (status_t error)
	{
		// driver error
		ErrorAlert(error);											// display an error message
		m_VerifyView->SendMessage(VerifyView::VV_VERIFYSTOPPED);
				// say to the view that the work is finished
		PostMessage(B_QUIT_REQUESTED);					// stop the thread
	}
	catch (int i)
	{
		// if int is -1, this means that the write has been cancelled by the user
		// otherwise, it is the address where a verify error occured
		if (i>=0)
		{
			// verify error. Tell the user
			char l_Message[201];
			snprintf(l_Message,200,"Verify error at address %x",i);
			(new BAlert("Verify error",l_Message,"Abort",NULL,NULL,B_WIDTH_AS_USUAL,B_STOP_ALERT))->Go();
		}
		m_VerifyView->SendMessage(VerifyView::VV_VERIFYSTOPPED);		
				// say to the view that the work is finished
		PostMessage(B_QUIT_REQUESTED);					// stop the thread
	}	
}
