/*
  BeNoPPP
  Write View
  (c) 2003 - Sylvain Tertois
*/

#include "WriteView.h"
#include "MainWindow.h"
#include "PicList.h"
#include "WriteThread.h"
#include "Preferences.h"
#include <Button.h>	
#include <TextControl.h>
#include <FilePanel.h>
#include <Path.h>
#include <MenuField.h>
#include <Menu.h>
#include <MenuItem.h>
#include <TextView.h>
#include <StatusBar.h>
#include <CheckBox.h>

// constructor
WriteView::WriteView(BRect r, MainWindow *win)
	: BView(r,NULL,B_FOLLOW_ALL_SIDES,B_WILL_DRAW)
{
	m_MainWindow = win;
	m_Created=false;
	m_Panel=NULL;
}

// destructor
WriteView::~WriteView()
{
	delete m_Panel;
}

static const float c_XMargin=5;
static const float c_YMargin=5;

// hook function called when the tab is activated
void WriteView::AttachedToWindow()
{
	if (!m_Created)
	{
		// first call, we must add the child views
		// first the file panel
		m_Panel = new BFilePanel(B_OPEN_PANEL,new BMessenger(this),NULL,
			B_FILE_NODE,false);

		// and then the view
		SetViewColor(ui_color(B_MENU_BACKGROUND_COLOR));
		
		// filename
		float l_X,l_Y;
		float l_Dummy,l_Height;
		/// first the "..." button
		m_FileButton = new BButton(BRect(0,c_YMargin,0,c_YMargin),NULL,"...",
			new BMessage(WV_FILEPANEL));
		AddChild(m_FileButton);
		m_FileButton->SetTarget(this);
		m_FileButton->GetPreferredSize(&l_Dummy,&l_Height);
		m_FileButton->ResizeTo(30,l_Height);
		l_X=Bounds().right-c_XMargin-m_FileButton->Frame().right;
		m_FileButton->MoveTo(l_X,c_YMargin);
		l_X-=c_XMargin;
		l_Y=m_FileButton->Frame().bottom+c_YMargin;
		
		/// and then the text field
		m_FileName = new BTextControl(BRect(c_XMargin,c_YMargin,l_X,c_YMargin),
			NULL,"File name:","",NULL);
		AddChild(m_FileName);
		m_FileName->SetTarget(this);
		m_FileName->SetModificationMessage(new BMessage(WV_FILENAMECHANGED));
		m_FileName->GetPreferredSize(&l_Dummy,&l_Height);
		m_FileName->ResizeTo(l_X-c_XMargin,l_Height);
		m_FileName->SetDivider(50);

		// PIC type menu
		BMenu *l_Menu = new BMenu("pic");
		for (unsigned int i_PicTypeNum=0; c_PicList[i_PicTypeNum].name != NULL; i_PicTypeNum++)
		{
			BMessage *l_Message = new BMessage(WV_PICTYPECHANGED);
			l_Message->AddInt32("pictypenum",i_PicTypeNum);
			l_Menu->AddItem(new BMenuItem(c_PicList[i_PicTypeNum].name,l_Message));
		}
		l_Menu->SetLabelFromMarked(true);
		l_Menu->SetTargetForItems(this);
		m_PicTypeField = new BMenuField(BRect(c_XMargin,l_Y,c_XMargin+150,c_YMargin),
			NULL,"Pic type:",l_Menu);
		AddChild(m_PicTypeField);
		m_PicTypeField->ResizeToPreferred();
		l_X=m_PicTypeField->Frame().right+c_XMargin;
		
		// verify checkbox
		m_VerifyBox=new BCheckBox(BRect(l_X,l_Y,l_X,l_Y),NULL,"Verify",
			new BMessage(WV_VERIFYCHANGED));
		AddChild(m_VerifyBox);
		m_VerifyBox->SetTarget(this);
		m_VerifyBox->ResizeToPreferred();
		
		l_Y=m_PicTypeField->Frame().bottom+c_YMargin;
		
		// message
		l_X=Bounds().right-c_XMargin;
		BTextView *l_TextView = new BTextView(BRect(c_XMargin,l_Y,l_X,l_Y+40),
			NULL,BRect(0,0,l_X-c_XMargin,40),B_FOLLOW_ALL_SIDES);
		l_TextView->SetText("Warning: do not insert the PIC in the programer before "
			"clicking on the button below!\n"
			"Insert it only when told to do so.");
		AddChild(l_TextView);
		l_TextView->SetViewColor(ui_color(B_MENU_BACKGROUND_COLOR));
		l_TextView->ResizeToPreferred();
		l_TextView->MakeEditable(false);
		l_TextView->MakeSelectable(false);
		l_Y=l_TextView->Frame().bottom+3*c_YMargin;

		// the 'Start' button		
		m_Start = new BButton(BRect(c_XMargin,l_Y,c_XMargin,l_Y),NULL,"Start",
			new BMessage(WV_START));
		AddChild(m_Start);
		m_Start->SetTarget(this);
		m_Start->SetEnabled(false);
		m_Start->ResizeToPreferred();
		l_Y=m_Start->Frame().bottom+c_YMargin;
		
		// the status bar
		m_StatusBar = new BStatusBar(BRect(c_XMargin,l_Y,Bounds().right-c_XMargin,l_Y),
			NULL,"Progress:","Idle");
		AddChild(m_StatusBar);
		m_StatusBar->GetPreferredSize(&l_Dummy,&l_Height);
		m_StatusBar->ResizeTo(Bounds().right-2*c_XMargin,l_Height);
			
		m_Created=true;
	}
	
	// this part of the function is executed each time the tab is activated. We update the controls in case they have been modified elsewhere
	// set the panel directory
	m_Panel->SetPanelDirectory(g_Preferences->m_HexPath.String());

	// set the pic type
	m_PicTypeField->Menu()->ItemAt(g_Preferences->m_PicTypeNum)->SetMarked(true);
	
	// set the verify state
	m_VerifyBox->SetValue(g_Preferences->m_VerifyWhenWrite?1:0);
}

// hook function called when a message arrives
void WriteView::MessageReceived(BMessage *message)
{
	switch (message->what)
	{
		case WV_FILEPANEL:
			// open the panel
			m_Panel->Show();
			break;
		
		case B_REFS_RECEIVED:
		{
			// the user closed the panel and confirmed the file name
			// first get the full file path
			entry_ref l_File;
			if (message->FindRef("refs",&l_File) == B_OK)
			{
				BPath l_FilePath(&l_File);
				
				// got it. change the text in the control
				m_FileName->SetText(l_FilePath.Path());
				m_MainWindow->PostMessage(WV_FILENAMECHANGED,this);
				
				// save the directory in the preferences
				BPath l_DirPath;
				l_FilePath.GetParent(&l_DirPath);
				g_Preferences->m_HexPath=l_DirPath.Path();
			}
			break;
		}
		
		case WV_PICTYPECHANGED:
			// the user changed the pic type
			// save the new type in the preferences
			message->FindInt32("pictypenum",(int32*)&g_Preferences->m_PicTypeNum);
			break;
		
		case WV_FILENAMECHANGED:
			// the user changed the file name
			// we can now activate the 'start' button, if it hasn't been already done
			m_Start->SetEnabled(true);
			break;
			
		case WV_VERIFYCHANGED:
			// the user changed the verify mode
			// save the new mode in the preferences
			g_Preferences->m_VerifyWhenWrite=(m_VerifyBox->Value()!=0);
			break;
			
		case WV_START:
			// the user clicked on 'start'
			// start a new thread that will do the work
			new WriteThread(new BMessenger(this),g_Preferences->m_PicTypeNum,
				m_FileName->Text(),g_Preferences->m_VerifyWhenWrite,
				g_Preferences->m_IOBase,g_Preferences->m_PinsConfig->m_Pins,
				g_Preferences->m_DetectHardware);
			break;
		
		case WV_WRITESTARTED:
			// the work just started. Vpp isn't on yet
			m_MainWindow->DisplayVppOff();
			break;
		
		case WV_VPPON:
			// vpp is now on
			m_MainWindow->DisplayVppOn();
			break;
		
		case WV_WRITESTOPPED:
			// the work is finished
			m_MainWindow->DisplayVppOff();
			break;
		
		case WV_STATUSMAX:
		{
			// the thread tells us how many times it will ask to update the status bar
			unsigned int l_MaxValue;
			if (message->FindInt32("max",(int32*)&l_MaxValue)==B_OK)
			{
				// reset the bar and put the "progress:" text
				m_StatusBar->Reset("Progress:");
				
				// set the max value to the one received
				m_StatusBar->SetMaxValue(l_MaxValue);
			}
			break;
		}
		
		case WV_STATUSUPDATE:
		{
			// called to show the progression of the work
			// if a text is inserted in the message, show it as well
			const char *l_UpdateText=NULL;
			message->FindString("text",&l_UpdateText);
			m_StatusBar->Update(1,NULL,l_UpdateText);
			break;
		}
			
		default:
			BView::MessageReceived(message);
	}
}
	
