<?xml version="1.0" encoding="iso-8859-1"?>
<!-- xsl file to convert a XML documentation to Netpositive compatible HTML-->

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output method="html" version="4.0" encoding="iso-8859-1" doctype-public="-//W3C//DTD HTML 4.01 Transitional//EN"/>

<xsl:variable name="longname"><xsl:choose>
	<xsl:when test="/doc/header/name/@long"><xsl:value-of select="/doc/header/name/@long"/></xsl:when>
	<xsl:when test="/doc/header/name/text()"><xsl:value-of select="/doc/header/name/text()"/></xsl:when>
	<xsl:otherwise><xsl:message terminate="yes">No long name!</xsl:message></xsl:otherwise>
</xsl:choose></xsl:variable>

<xsl:variable name="shortname"><xsl:choose>
	<xsl:when test="/doc/header/name/@short"><xsl:value-of select="/doc/header/name/@short"/></xsl:when>
	<xsl:when test="/doc/header/name/text()"><xsl:value-of select="/doc/header/name/text()"/></xsl:when>
	<xsl:otherwise><xsl:message terminate="yes">No short name!</xsl:message></xsl:otherwise>
</xsl:choose></xsl:variable>

<!-- base -->
<xsl:template match="doc">
	<html>
		<head>
			<title>
				<xsl:value-of select="$shortname"/>
				<xsl:if test="header/version"> - <xsl:apply-templates select="header/version"/></xsl:if>
			</title>
		</head>
		<body bgcolor="#F0F0F0" color="#000000" link="#2222AA">
			<h1 align="center"><font size="+5"><xsl:value-of select="$longname"/>
				<xsl:if test="header/version"> - <xsl:apply-templates select="header/version"/></xsl:if>
			</font></h1>
			<h2 align="center">Copyright <xsl:text disable-output-escaping="yes">&amp;copy;</xsl:text> <xsl:apply-templates select="header/date"/> <xsl:apply-templates select="header/author"/></h2>
			<xsl:if test="header/site"><h3 align="center">Main site: <a href="{header/site/text()}"><xsl:apply-templates select="header/site"/></a></h3></xsl:if>
			<hr noshade="noshade"/>
			<h2><a name="TOC"><font color="#430000"><font size="6">T</font>able of contents</font></a></h2>
			<xsl:apply-templates select="contents/section[title]" mode="toc"/>
			<xsl:apply-templates select="contents"/>
			<hr noshade="noshade"/>
			<center><i><font color="#555555"><xsl:value-of select="$longname"/> documentation</font></i></center>
		</body>
	</html>
</xsl:template>

<xsl:template match="author">
	<xsl:text> </xsl:text><a><xsl:if test="@email"><xsl:attribute name="href"><xsl:value-of select="concat('mailto:',@email)"/></xsl:attribute></xsl:if>
	<xsl:value-of select="@name"/></a>
</xsl:template>

<xsl:template match="section" mode="toc">
	<xsl:variable name="number"><xsl:apply-templates select="ancestor-or-self::*" mode="number"/></xsl:variable>
	<a href="{concat('#sec',$number)}">
		<xsl:if test="@counters!='no'"><xsl:value-of select="concat($number,'. ')"/></xsl:if>
		<xsl:apply-templates select="title" mode="toc"/>
	</a><br/>
	<xsl:apply-templates select="section[title]" mode="toc"/>
</xsl:template>


<xsl:template match="section[title]" mode="number">
	<xsl:if test="name(..)='section'">.</xsl:if><xsl:value-of select="count(preceding-sibling::section[title])+1"/>
</xsl:template>

<xsl:template match="*" mode="number"/>

<xsl:template match="contents">
	<xsl:apply-templates mode="contents"/>
</xsl:template>

<xsl:template match="section" mode="contents">
	<xsl:if test="name(..)!='section'"><hr noshade="noshade"/></xsl:if>
	<xsl:apply-templates mode="title"/>
	<xsl:apply-templates mode="contents"/>
</xsl:template>

<xsl:template match="title" mode="title">
	<xsl:variable name="level"><xsl:value-of select="count(ancestor::section)"/></xsl:variable>
	<xsl:variable name="htmlcode"><xsl:choose>
		<xsl:when test="$level=1">h2</xsl:when>
		<xsl:when test="$level=2">h3</xsl:when>
		<xsl:when test="$level=3">h4</xsl:when>
		<xsl:when test="$level=4">h5</xsl:when>
		<xsl:otherwise>h6</xsl:otherwise>
	</xsl:choose></xsl:variable>
	<xsl:variable name="number"><xsl:apply-templates select="ancestor::*" mode="number"/></xsl:variable>
	<xsl:element name="{$htmlcode}">
		<a name="{concat('sec',$number)}">
			<xsl:choose>
				<xsl:when test="$level=1"><font color="#430000">
					<xsl:value-of select="$number"/>.
					<font size="6"><xsl:value-of select="substring(text(),1,1)"/></font><xsl:value-of select="substring(text(),2)"/>
				</font></xsl:when>
				<xsl:otherwise><xsl:value-of select="concat($number,'. ',text())"/></xsl:otherwise>
			</xsl:choose>
		</a>
	</xsl:element>
</xsl:template>

<xsl:template match="title" mode="contents"/>
<xsl:template match="*" mode="title"/>

<xsl:template match="para" mode="contents">
	<p>
		<xsl:apply-templates  mode="contents"/>
	</p>
</xsl:template>

<xsl:template match="link" mode="contents">
	<a href="{@url}"><xsl:choose>
		<xsl:when test="text()"><xsl:value-of select="text()"/></xsl:when>
		<xsl:otherwise><xsl:value-of select="@url"/></xsl:otherwise>
	</xsl:choose></a>
</xsl:template>

<xsl:template match="filename" mode="contents">
	<tt><b><xsl:apply-templates mode="contents"/></b></tt>
</xsl:template>

<xsl:template match="code" mode="contents">
	<tt><xsl:apply-templates mode="contents"/></tt>
</xsl:template>

<xsl:template match="image" mode="contents">
	<center><img src="{@file}" alt="{text()}"/></center>
</xsl:template>

<xsl:template match="list" mode="contents">
	<ul>
		<xsl:apply-templates select="elem" mode="contents"/>
	</ul>
</xsl:template>

<xsl:template match="elem" mode="contents">
	<li>
		<xsl:apply-templates mode="contents"/>
	</li>
</xsl:template>

</xsl:stylesheet>
