/*
  BeNoPPP
  Driver control codes
  (c) 2003 - Sylvain Tertois
*/

#ifndef CONTROLCODES_H
#define CONTROLCODES_H

#include <Drivers.h>

#ifdef __cplusplus
extern "C" {
#endif

// ioctl instructions
enum {
	BNPPP_SETBASEPORT = B_DEVICE_OP_CODES_END+1001,
			// set the base port number to access the parralel port (default 0x378)
			// data is a int (4 bytes)
	BNPPP_DATAWRITABLE,
			// prepare to write data
	BNPPP_DATAREADABLE,
			// prepare to read data
	BNPPP_DATADOWN,
			// send a 0
	BNPPP_DATAUP,
			// send a 1
	BNPPP_CLOCKDOWN, BNPPP_CLOCKUP,
			// clock
	BNPPP_VPPON,
			// apply vpp
	BNPPP_VPPOFF,
			// stop applying vpp
	BNPPP_DATAIN,
			// read data
			// (result in data, 1 byte, in lsb)
	BNPPP_ALLPINSLOW,
			// set all pins to 0V
	BNPPP_SETPINS,
			// set the pins to use (data must be a UsePin[NUMFUNCTIONS])
	BNPPP_VDDON,
			// apply vdd
	BNPPP_VDDOFF
			// stop applying vdd
	};
	
typedef enum _function { F_WRITE = 0, F_CLK, F_MCLR, F_PGM, F_READ, F_VDD, F_IN1,F_IN2, NUMFUNCTIONS } Function;
// functions list:
//	F_WRITE: bit to write to PIC
//	F_CLK: PIC clock
//	F_MCLR: 1: apply 0V to MCLR (-> reset the PIC) 0: do not apply 0V (-> 5V or 12V)
//	F_PGM: when F_MCLR is 0: 0: apply 5V, 1: apply 12V
//	F_READ: bit to read from PIC
//	F_VDD: apply VDD (5V)
//	F_IN1/2: 0: write to PIC, 1: read from PIC

// note: the state 5V to MCLR (F_MCLR to 0 and F_PGM to 0) doesn't have to be supported by the hardware

typedef enum _pin { P_NONE=0,
		// pins for parallel port - outputs
		P_STROBE, P_D0, P_D1, P_D2, P_D3, P_D4, P_D5, P_D6, P_D7, P_AUTOFD,
		P_INIT,P_SELIN,
		// - inputs
		P_ACK,P_BUSY,P_PERROR,P_SEL,P_ERROR,
		// pins for serial port - outputs
		P_TXD,P_DTR,P_RTS,
		// - inputs
		P_CTS, P_CD, P_DSR, P_RI,
		// end of list
		NUMPINS } Pin;

// structure that describes a pin to use for a function
typedef struct _usepin {
		Pin pin;
		int inverted;	// if non zero, inverse the pin
		} UsePin;
	
#ifdef __cplusplus
}			// extern "C" {
#endif

#endif // CONTROLCODES_H
